@@ -12,13 +12,18 @@
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <netinet/in.h>
 #include "thc-ipv6.h"
+#include "fps.h"
 
 #define MAX_ALIVE   65536
 #define MAX_NETS    1024
 #define MAX_VENDID  64
 #define MAX_PORTS   256
 #define MAX_FOUR    16
+#define TCP_OPT_LEN 28   // 20
+#define FPS_INVALID "invalid packet data"
+#define FPS_UNKNOWN "unknown"
 #define RESP_PONG   "ICMP echo-reply"
 #define RESP_UNREACH_PORT    "ICMP port unreachable"
 #define RESP_UNREACH_ROUTE   "ICMP network unreachable"
@@ -39,8 +44,14 @@
 #define RESP_OTHER  "TCP misc-options"
 #define RESP_UNKNOWN "unknown"
 
+struct fingerprint {
+  char OS[1024];
+  char FP[200];
+};
+
+static char _fingerprint[4096]; // not thread safe!
 unsigned char buf[8], *alive[MAX_ALIVE], *tagging = NULL;
-int alive_no = 0, resolve = 0, waittime = 1, rawmode = 0, portscan = 0, curr = 0, list = 0;
+int alive_no = 0, resolve = 0, waittime = 1, portscan = 0, curr = 0, list = 0, slow = 0;
 int synports[MAX_PORTS], ackports[MAX_PORTS], udpports[MAX_PORTS];
 int ndp_only = 0, do_ping = 1, do_dst = 1, do_hop = 0, verbose = 0, srcport = -1, do_help = 0, do_hopcount = 0, still_not_there = 0, rst_means_alive = 1;
 unsigned long int tcount = 0;
@@ -48,9 +59,11 @@ FILE *out = NULL;
 struct hostent *he = NULL;
 short int si, sp, sp2;
 
+
+
 // all dict entries must start with a single from/to 0,0,0,0
 // and end with a single from/to ffff,ffff,ffff,ffff
-unsigned short int dict[] = { 0, 0, 0, 0, /*to */ 0, 0, 0, 0,
+unsigned short int dict_small[] = { 0, 0, 0, 0, /*to */ 0, 0, 0, 0,
   0, 0, 0, 1, /*to */ 0, 0, 0, 0x2ff,   // 1975 tests
   0, 0, 0, 0x300, /*to */ 0, 0, 0, 0x305,
   0, 0, 0, 0x400, /*to */ 0, 0, 0, 0x405,
@@ -93,6 +106,7 @@ unsigned short int dict[] = { 0, 0, 0, 0, /*to */ 0, 0, 0, 0,
   0, 0, 0, 0xfff0, /*to */ 0, 0, 0, 0xffff,
   0, 0, 1, 0, /*to */ 0, 0, 1, 0x1ff,
   0, 0, 2, 0, /*to */ 0, 0, 0x1bb, 5,
+  0, 0, 2, 6, /*to */ 0, 0, 9, 9,
   0, 0, 2, 0xa, /*to */ 0, 0, 2, 0x20,
   0, 0, 2, 0x21, /*to */ 0, 0, 3, 0x21,
   0, 0, 2, 0x22, /*to */ 0, 0, 3, 0x22,
@@ -102,8 +116,7 @@ unsigned short int dict[] = { 0, 0, 0, 0, /*to */ 0, 0, 0, 0,
   0, 0, 2, 0x80, /*to */ 0, 0, 9, 0x80,
   0, 0, 2, 0x1bb, /*to */ 0, 0, 9, 0x1bb,
   0, 0, 2, 0x500, /*to */ 0, 0, 9, 0x500,
-  0, 0, 2, 6, /*to */ 0, 0, 9, 9,
-  0, 0, 0xa, 0, /*to */ 0, 0, 0xf, 2,
+//  0, 0, 0xa, 0, /*to */ 0, 0, 0xf, 2,
   0, 0, 0x80, 6, /*to */ 0, 0, 0x80, 0x1f,
   0, 0, 0x200, 0, /*to */ 0, 0, 0x200, 3,
   0, 0, 0x389, 0, /*to */ 0, 0, 0x389, 3,
@@ -115,7 +128,7 @@ unsigned short int dict[] = { 0, 0, 0, 0, /*to */ 0, 0, 0, 0,
   0, 0, 0x8080, 0, /*to */ 0, 0, 0x8080, 3,
   0, 0, 0xdead, 0xbeef, /*to */ 0, 0, 0xdead, 0xbeef,
 //  0, 1, 0, 0, /*to */ 0, 3, 3, 3,
-  0, 0, 0, 0, /*to */ 4, 4, 4, 4,  // some doubles here
+  0, 0, 0, 0, /*to */ 4, 4, 4, 4,  // 24 doubles here == 0.42%
   1, 0, 0, 5, /*to */ 1, 0, 0, 0xf,
 //  2, 0, 1, 0, /*to */ 2, 0, 1, 3,
   2, 0, 0, 5, /*to */ 2, 0, 0, 0xd,
@@ -126,17 +139,70 @@ unsigned short int dict[] = { 0, 0, 0, 0, /*to */ 0, 0, 0, 0,
   0xffff, 0xffff, 0xffff, 0xffff, /*to */ 0xffff, 0xffff, 0xffff, 0xffff
 };
 
+unsigned short int dict_large[] = { 0, 0, 0, 0, /*to */ 0, 0, 0, 0,
+  0, 0, 0, 1, /*to */ 0, 0, 1, 0x2fff,   // 1975 tests
+  0, 0, 0, 0x3000, /*to */ 0, 0, 1, 0x3333,
+  0, 0, 0, 0x5353, /*to */ 0, 0, 2, 0x5353,
+  0, 0, 0, 0x6666, /*to */ 0, 0, 2, 0x6667,
+  0, 0, 0, 0x8000, /*to */ 0, 0, 2, 0x8000,
+  0, 0, 0, 0x8080, /*to */ 0, 0, 2, 0x8080,
+  0, 0, 0, 0xaaaa, /*to */ 0, 0, 2, 0xaaaa,
+  0, 0, 0, 0xabcd, /*to */ 0, 0, 2, 0xabcd,
+  0, 0, 0, 0xbabe, /*to */ 0, 0, 2, 0xbabe,
+  0, 0, 0, 0xbeef, /*to */ 0, 0, 2, 0xbeef,
+  0, 0, 0, 0xcafe, /*to */ 0, 0, 2, 0xcafe,
+  0, 0, 0, 0xc0de, /*to */ 0, 0, 2, 0xc0de,
+  0, 0, 0, 0xdead, /*to */ 0, 0, 2, 0xdead,
+  0, 0, 0, 0xf500, /*to */ 0, 0, 2, 0xf500,
+  0, 0, 0, 0xfeed, /*to */ 0, 0, 2, 0xfeed,
+  0, 0, 0, 0xfff0, /*to */ 0, 0, 2, 0xffff,
+  0, 0, 2, 0, /*to */ 0, 0, 0x1bb, 5,
+  0, 0, 2, 0x11, /*to */ 0, 0, 2, 0x100,
+  0, 0, 2, 0x1bb, /*to */ 0, 0, 9, 0x1bb,
+  0, 0, 2, 0x500, /*to */ 0, 0, 9, 0x500,
+  0, 0, 2, 6, /*to */ 0, 0, 10, 10,
+  0, 0, 0xa, 0, /*to */ 0, 0, 0xf, 5,
+  0, 0, 0x80, 6, /*to */ 0, 0, 0x80, 0x1f,
+  0, 0, 0x200, 0, /*to */ 0, 0, 0x200, 5,
+  0, 0, 0x389, 0, /*to */ 0, 0, 0x389, 5,
+  0, 0, 0x443, 0, /*to */ 0, 0, 0x443, 5,
+  0, 0, 0x500, 0, /*to */ 0, 0, 0x500, 5,
+  0, 0, 0x666, 0, /*to */ 0, 0, 0x669, 5,
+  0, 0, 0x3128, 0, /*to */ 0, 0, 0x3128, 5,
+  0, 0, 0x6666, 0, /*to */ 0, 0, 0x6669, 5,
+  0, 0, 0x8080, 0, /*to */ 0, 0, 0x8080, 5,
+  0, 0, 0xdead, 0xbeef, /*to */ 0, 0, 0xdead, 0xbeef,
+//  0, 1, 0, 0, /*to */ 0, 3, 3, 3,
+  0, 0, 0, 0, /*to */ 5, 5, 5, 5,  // some doubles here
+  1, 0, 0, 6, /*to */ 1, 0, 0, 0x10,
+//  2, 0, 1, 0, /*to */ 2, 0, 1, 3,
+  2, 0, 0, 5, /*to */ 2, 0, 0, 0x10,
+//  1, 2, 3, 4, /*to */ 1, 2, 3, 4,
+  6, 0, 0, 0, /*to */ 0xff, 0, 0, 2,
+  0xffff, 0x00ff, 0xfe00, 0xfffe, /*to */ 0xffff, 0x00ff, 0xfe00, 0xffff,
+  0xffff, 0xffff, 0xffff, 0xfffe, /*to */ 0xffff, 0xffff, 0xffff, 0xfffe,
+  0xffff, 0xffff, 0xffff, 0xffff, /*to */ 0xffff, 0xffff, 0xffff, 0xffff
+};
+
+unsigned short int *dict = NULL;
+
+unsigned char tcp_opt[TCP_OPT_LEN] = { 
+  0x02, 0x04, 0xff, 0xff, 0x04, 0x02, 0x08, 0x0a, 0x00, 0x09,
+  0x00, 0x09, 0x00, 0x00, 0x00, 0x00, 0x01, 0x03, 0x03, 0xff,
+  0x06, 0x06, 0xb0, 0x0b, 0xba, 0xbe, 0x01, 0x01
+};
+
 // more keywords:
 // cafe, dead, beef, affe, b00b, babe, f00, fefe, ffff, 1337, 666, 0, 1
 
 void help(char *prg) {
-  printf("%s %s (c) 2013 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
-  printf ("Syntax: %s [-CMPSdlpv] [-I srcip6] [-i file] [-o file] [-e opt] [-s port,..] [-a port,..] [-u port,..] [-W TIME] interface [unicast-or-range-or-multicast-address]\n\n", prg);
+  printf("%s %s (c) 2014 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf ("Syntax: %s [-CFHLMPSdlpvV] [-I srcip6] [-i file] [-o file] [-e opt] [-s port,..] [-a port,..] [-u port,..] [-T tag] [-W TIME] interface [unicast-or-multicast-address [remote-router]]\n\n", prg);
   printf("Options:\n");
   printf("  -i file    check systems from input file\n");
   printf("  -o file    write results to output file\n");
   printf("  -M         enumerate hardware addresses (MAC) from input addresses (slow!)\n");
-  printf("  -C         enumerate common address space from input addresses\n");
+  printf("  -C         enumerate common addresses of input networks, x2 for large scan\n");
   printf("  -4 ipv4/range    test various IPv4 address encodings per network (1.2.3.4/24)\n");
   printf("  -p         send a ping packet for alive check (default)\n");
   printf("  -e dst,hop send an errornous packets: destination (default), hop-by-hop\n");
@@ -149,35 +215,150 @@ void help(char *prg) {
     printf("  -W time    time in ms to wait after sending a packet (default: %d)\n", waittime);
     printf("  -S         slow mode, get best router for each remote target or when proxy-NA\n");
     printf("  -I srcip6  use the specified IPv6 address as source\n");
-    printf("  -l         use link-local address instead of global address\n");
-    printf("  -P         only print addresses that would be scanned, no scanning\n");
+    printf("  -l         use link-local address for multicast addresses instead of global\n");
+    printf("  -F         firewall mode: -p -e dst,hop -u 53 -s 22,25,80,443,9511 -a 9511\n");
+    printf("  -P         only print addresses that would be scanned, no packets are sent!\n");
     printf("  -v         verbose (twice: detailed, thrice: dumping packets)\n");
+    if (do_help > 1) {
+      printf("  -r         renew IPv6 src address for every new target (waits if none there)\n");
+      printf("  -L         local mode - perform only NDP and report local systems alive\n");
+      printf("  -R         do not consider TCP-RST as alive (good with firewalls, e.g. -F)\n");
+      printf("  -H         print hop count of received packets\n");
+      printf("  -T tag     put tag string in ICMP packets\n");
+      printf("  -x port    TCP/UDP src port for -s, -a and -u\n");
+      printf("  -y step    for range scans (2000::0-f), define the step range (default: 1)\n");
+      printf("  -Z mac     use given destination mac address\n");
+      printf("  -V         enable debug output\n");
+    } else
+      printf("  -hh        show more options\n");
   } else
-    printf("  -h         to display more command line options and help\n");
+    printf("  -h         to display more command line options and help (-hh: more options)\n");
   printf("\nTarget address on command line or in input file can include ranges in the form\n");
   printf("of 2001:db8::1-fff or 2001:db8::1-2:0-ffff:0:0-ffff, etc.\n");
   if (do_help) {
     printf("Do not use the ranges (from-to) option with -M, -C or -4.\n");
-    printf("If you specify a remote router, fragmentation+srcroute is performed.\n");
+    printf("If you use SYN packets (-s/-F option), automatic OS detection is performed.\n");
+//    printf("If you specify a remote router, fragmentation+srcroute is performed.\n");
     printf("Returns -1 on errors, 0 if a system was found alive or 1 if nothing was found.\n");
   }
   exit(-1);
 }
 
+/*****************************************************************************************/
+
+/*
+    Copyright (c) warlord @ nologin.org.  All rights reserved.
+    For more information, please visit http://www.nologin.org
+ */
+
+char *get_OS(char *query_fp) {
+  int i = 0;
+
+  for (i = 0; i < sizeof(fingerprintsArray) / sizeof(fingerprintsArray[0]); i++) {
+    if (strcmp(fingerprintsArray[i].fingerprint, query_fp) == 0)
+      return fingerprintsArray[i].OS;
+  }
+
+  return FPS_UNKNOWN;
+}
+
+char *warlord_checkFingerprint(char *buffer, int len) {
+  char *os, *end, *ptr, ip_mod, ip_ver = 0, ip_hdr_size = 0;
+
+  ip_ver = (((unsigned char)buffer[0] & 0xf0) >> 4);
+  if (ip_ver == 4) {
+    ip_mod = 0;
+    ip_hdr_size = ((buffer[0] & 0x0f) << 2);
+  } else if (ip_ver == 6) {
+    ip_mod = 20;		// to align packet sizes in FPS with IPv4 for IPv6 (would be 40 bytes - 20 = 20 equals to IPv4)
+    ip_hdr_size = 40;
+  }
+
+  if (ip_ver == 0 || (len - ip_hdr_size) < 20 || ip_hdr_size < 20 || ip_hdr_size > 40) // invalid ip version or packet too short?
+    return FPS_INVALID;
+
+  snprintf(_fingerprint, sizeof(_fingerprint) - 1, "%04x:%02x:%04x",
+           len + 20 - ip_hdr_size, // total length (calculated with assumed ipv4 header
+           (unsigned char) buffer[ip_hdr_size + 12],        // header length
+           ntohs(*(in_port_t *) &buffer[ip_hdr_size + 14]));       // window size
+
+  //So what kind of tcp options did we receive?
+  //This is being used for OS fingerprinting
+  end = &buffer[len];
+  if (len > ip_hdr_size + (unsigned char) buffer[ip_hdr_size + 12])
+    end = &buffer[ip_hdr_size + (unsigned char) buffer[ip_hdr_size + 12]];
+  for (ptr = &buffer[ip_hdr_size + 20]; ptr < end; ) {
+    switch (*ptr) {
+    case 0x0:			// end of options
+      ptr = end;
+      break;
+    case 0x1:                  // some pad entire options portion with NOP to keep response option size the same
+      strncat(_fingerprint, ":NOP", sizeof(_fingerprint) - strlen(_fingerprint) - 1);
+      ptr++;
+      break;
+    case 0x2:                  // segment size
+      snprintf(&_fingerprint[strlen(_fingerprint)], sizeof(_fingerprint) - strlen(_fingerprint), ":SS%04x", ntohs(*(in_port_t *) (ptr + 2)));
+      ptr += 4;
+      break;
+    case 0x3:                  // window scaling
+      strncat(_fingerprint, ":WSxx", sizeof(_fingerprint) - strlen(_fingerprint) - 1);
+      ptr += 3;
+      break;
+    case 0x4:                  // Sack Permitted / Sack Denied
+      switch (ptr[1]) {
+      case 0x2:
+        strncat(_fingerprint, ":SP", sizeof(_fingerprint) - strlen(_fingerprint) - 1);
+        break;
+      default:
+        strncat(_fingerprint, ":SD", sizeof(_fingerprint) - strlen(_fingerprint) - 1);
+        break;
+      }
+      ptr += 2;
+      break;
+    case 0x6:			// echo request
+      strncat(_fingerprint, ":PI", sizeof(_fingerprint) - strlen(_fingerprint) - 1);
+      ptr += 6;
+      break;
+    case 0x7:			// echo reply
+      strncat(_fingerprint, ":PO", sizeof(_fingerprint) - strlen(_fingerprint) - 1);
+      ptr += 6;
+      break;
+    case 0x8:                  // Time stamp
+      strncat(_fingerprint, ":TS", sizeof(_fingerprint) - strlen(_fingerprint) - 1);
+      ptr += 10;
+      break;
+    default:                   // unknown
+      snprintf(&_fingerprint[strlen(_fingerprint)], sizeof(_fingerprint) - strlen(_fingerprint), ":UOP%02x", (unsigned char)(*ptr));
+      ptr += (unsigned char)ptr[1];
+      break;
+    }
+  }
+  _fingerprint[sizeof(_fingerprint) - 1] = 0;
+//              printf ("%s\t\tport %5d\t", inet_ntoa (src), ntohs (recvtcp->source));
+  os = get_OS(_fingerprint);
+
+  if (strcmp(os, FPS_UNKNOWN) == 0)
+    return _fingerprint;
+
+  return os;
+}
+
+/* end of warlords code */
+/**************************************************************************/
+
 void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned char *data) {
   int i, ok = 0, len = header->caplen, offset = 0, nxt;
   unsigned char *ptr = (unsigned char *) data, *p1, *p2, *p3, sport[16] = "", *orig_dst = NULL;
-  char *type = RESP_UNKNOWN, hopcount[20] = "";
+  char *type = RESP_UNKNOWN, hopcount[20] = "", *os = NULL;
 
-  if (!rawmode) {
-    ptr += 14;
-    len -= 14;
-  }
   if (do_hdr_size) {
-    ptr += (do_hdr_size - 14);
-    len -= (do_hdr_size - 14);
+    ptr += do_hdr_size;
+    len -= do_hdr_size;
     if ((ptr[0] & 240) != 0x60)
       return;
+  } else {
+    ptr += 14;
+    len -= 14;
   }
 
   if (debug)
@@ -248,6 +429,7 @@ void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned
       snprintf(sport, sizeof(sport), "%d/", i);
       switch (ptr[53 + offset]) {
       case (TCP_SYN + TCP_ACK):
+        os = warlord_checkFingerprint(ptr, len);
         type = RESP_SYNACK;
         break;
       case TCP_ACK:
@@ -310,9 +492,11 @@ void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned
       ok = 0;
     }
     if (ok == 0) {
-      type = RESP_ERROR;
-      snprintf(sport, sizeof(sport), "%d:%d/", ptr[40], ptr[41]);
-      ok = 2;
+      if (slow == 0 || ptr[40] != ICMP6_NEIGHBORADV) {
+        type = RESP_ERROR;
+        snprintf(sport, sizeof(sport), "%d:%d/", ptr[40], ptr[41]);
+        ok = 2;
+      }
     } else
       orig_dst = thc_ipv62notation(ptr + 72 + offset);
   }
@@ -328,12 +512,12 @@ void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned
   if (ok) {
     if (do_hopcount)
       sprintf(hopcount, " {hop count: %d}", ptr[7]);
-    if (portscan == 0 || (portscan && (verbose > 2 || (type != RESP_UNREACH_PORT && type != RESP_UNREACH_FW)))) {
+    if (portscan == 0 || (portscan && (verbose > 2 || (strcmp(type, RESP_UNREACH_PORT) != 0 && strcmp(type, RESP_UNREACH_FW) != 0)))) {
       if (resolve)
         he = gethostbyaddr(ptr + 8, 16, AF_INET6);
       p2 = thc_ipv62notation(ptr + 8);
-      printf("Alive: %s%s%s%s [%s%s%s%s]%s\n", p2, resolve ? " (" : "", resolve
-             && he != NULL ? he->h_name : "", resolve ? ")" : "", sport, type, orig_dst != NULL ? " for " : "", orig_dst != NULL ? (char *) orig_dst : "", hopcount);
+      printf("Alive: %s%s%s%s [%s%s%s%s]%s%s%s%s\n", p2, resolve ? " (" : "", resolve
+             && he != NULL ? he->h_name : "", resolve ? ")" : "", sport, type, orig_dst != NULL ? " for " : "", orig_dst != NULL ? (char *) orig_dst : "", hopcount, os == NULL ? "" : " (OS: ", os == NULL ? "" : os, os == NULL ? "" : ")" );
       if (out != NULL)
         fprintf(out, "%s%s%s%s\n", p2, resolve ? " (" : "", (resolve && he != NULL) ? he->h_name : "", resolve ? ")" : "");
       free(p2);
@@ -379,7 +563,7 @@ void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned
 
 void get_ports_from_cmdline(int ports[], char *plist, char param) {
   int p, c = 0;
-  char mylist[strlen(plist + 1)], *ptr, *ptr2;
+  char mylist[strlen(plist) + 1], *ptr, *ptr2;
 
   if (strtok(plist, "0123456789,") != NULL) {
     fprintf(stderr, "Error: ports must be defined by numbers and separated by a comma, e.g. \"-%c 22,53,80\"\n", param);
@@ -486,16 +670,16 @@ int adress4to6(unsigned char *addr6, unsigned int addr4, char *state) {
 
 int main(int argc, char *argv[]) {
   unsigned char string[128]; // = "ip6 and dst ";
-  unsigned char *pkt = NULL, *router6 = NULL, *cur_dst, *p2, *p3, *smac, buf2[6];
+  unsigned char *pkt = NULL, *router6 = NULL, *cur_dst, *p2, *p3, *ptr3, *smac, buf2[6];
   unsigned char *multicast6 = NULL, *src6 = NULL, *mac = NULL, *rmac = NULL, *routers[2];
-  int pkt_len = 0, prefer = PREFER_GLOBAL, fromto = 0, dictptr = 0, offset = 14;
+  int pkt_len = 0, prefer = PREFER_GLOBAL, fromto = 0, dictptr = 0, offset = 14, step = 1;
   int enumerate_mac = 0, enumerate_dhcp = 0, i, j, k, l, cur_enum = 0, print_only = 0;
-  int slow = 0, no_vendid = 0, no_nets = 0, local = -1, no_send = 1, no_send_local = 1, no_send_remote = 2, nos = 0;
-  char *interface = NULL, *input = NULL, *output = NULL, line[128], line2[128], *ptr, *ptr2, *ptr3, do_router = 0, ok;
+  int no_vendid = 0, no_nets = 0, local = -1, no_send = 1, no_send_local = 1, no_send_remote = 2, nos = 0, renew = 1, errcnt, sendrc;
+  char *interface = NULL, *input = NULL, *output = NULL, line[128], line2[128], *ptr, *ptr2, do_router = 0, ok;
   unsigned int four_from[MAX_FOUR], four_to[MAX_FOUR], addr_cur;
-  unsigned char fcnt = 0, bh, bm, bl, restart, use_dmac = 0, dump_all = 0;
-  unsigned short int ip1, ip2, ip3, ip4, cip1, cip2, cip3, cip4, cip5, cip6, cip7, cip8;
-  unsigned short int fip1, fip2, fip3, fip4, fip5, fip6, fip7, fip8, tip1, tip2, tip3, tip4, tip5, tip6, tip7, tip8;
+  unsigned char fcnt = 0, bh, bm, bl, restart, use_dmac = 0, dump_all = 0, inc_next = 0, inc_step = 1;
+  unsigned int ip1, ip2, ip3, ip4, cip1, cip2, cip3, cip4, cip5, cip6, cip7, cip8;
+  unsigned int fip1, fip2, fip3, fip4, fip5, fip6, fip7, fip8, tip1, tip2, tip3, tip4, tip5, tip6, tip7, tip8;
   unsigned char vendid[MAX_VENDID][11], nets[MAX_NETS][8], orig_dst[16], dmac[27] = { 0, 0, 0, 0, 0, 0, 0 };
   in_addr_t addr4;
 //  unsigned char dns4buf[] = { 0xde, 0xad, 0x01, 0x00, 0x00, 0x01, 0x00, 0x00,
@@ -518,11 +702,10 @@ int main(int argc, char *argv[]) {
     help(argv[0]);
 
   j = 0;
-  while ((i = getopt(argc, argv, "RhH4:W:w:PSLFdrlMDCn:i:o:pvs:a:u:e:VZ:I:Xx:T:")) >= 0) {
+  while ((i = getopt(argc, argv, "CRhH4:W:w:PSLFdrlMDn:i:o:pvs:a:u:e:VZ:I:Xx:y:T:")) >= 0) {
     switch (i) {
     case 'h':
-      do_help = 1;
-      help(argv[0]);
+      do_help++;
       break; // not reached
     case 'H':
       do_hopcount = 1;
@@ -552,8 +735,9 @@ int main(int argc, char *argv[]) {
           exit(-1);
         }
         strcpy(ptr2, optarg);
-        ptr = index(ptr2, '/');
-        *ptr++ = 0;
+        k = 32;
+        if ((ptr = index(ptr2, '/')) != NULL)
+          *ptr++ = 0;
         if ((k = atoi(ptr)) < 8 || k > 31) {
           fprintf(stderr, "Error: network size may only be between /8 and /31: %s\n", optarg);
           exit(-1);
@@ -608,8 +792,7 @@ int main(int argc, char *argv[]) {
       resolve = 1;
       break;
     case 'r':
-      thc_ipv6_rawmode(1);
-      rawmode = 1;
+      renew = 1;
       break;
     case 'l':
       prefer = PREFER_LINK;
@@ -617,9 +800,13 @@ int main(int argc, char *argv[]) {
     case 'M':
       enumerate_mac = 1;
       break;
-    case 'D':
     case 'C':
+    case 'D':
       enumerate_dhcp = 1;
+      if (dict == NULL)
+        dict = dict_small;
+      else
+        dict = dict_large;
       break;
     case 'n':
       no_send_local = no_send_remote = atoi(optarg);
@@ -688,12 +875,26 @@ int main(int argc, char *argv[]) {
       break;
     case 'x':
       srcport = atoi(optarg);
+      if (srcport < 0 || srcport > 65535) {
+        fprintf(stderr, "Error: invalid port: %s\n", optarg);
+        exit(-1);
+      }
+      break;
+    case 'y':
+      step = atoi(optarg);
+      if (step < 1 || step > 256) {
+        fprintf(stderr, "Error: invalid step range (valid: 1-256): %s\n", optarg);
+        exit(-1);
+      }
       break;
     default:
       fprintf(stderr, "Error: unknown option -%c\n", i);
       exit(-1);
     }
   }
+  
+  if (do_help)
+    help(argv[0]);
 
   if (slow && ndp_only) {
     fprintf(stderr, "Error: you can not use the -S and -L options togther!\n");
@@ -770,20 +971,27 @@ int main(int argc, char *argv[]) {
     }
   }
   //strcat(string, thc_ipv62notation(src6));
-  sprintf(string, "dst %s", thc_ipv62notation(src6));
-  if (dump_all == 0) {
-    if (portscan || synports[0] != -1 || udpports[0] != -1 || ackports[0] != -1) {
-      strcat(string, " and ( icmp6 or ");
-      if (udpports[0] != -1)
-        strcat(string, "udp ");
-      if (udpports[0] != -1 && (portscan || synports[0] != -1 || ackports[0] != -1))
-         strcat(string, "or ");
-      if (portscan || synports[0] != -1 || ackports[0] != -1)
-        strcat(string, "tcp ");
-      strcat(string, ")");
-    } else
-      strcat(string, " and icmp6");
-  }
+  //thc_dump_data(src6, 16, "SRC6");
+  if (renew == 0) {
+    sprintf(string, "dst %s", thc_ipv62notation(src6));
+    if (dump_all == 0) {
+      if (renew == 0)
+        strcat(string, " and ");
+      if (portscan || synports[0] != -1 || udpports[0] != -1 || ackports[0] != -1) {
+        strcat(string, "( icmp6 or ");
+        if (udpports[0] != -1)
+          strcat(string, "udp ");
+        if (udpports[0] != -1 && (portscan || synports[0] != -1 || ackports[0] != -1))
+           strcat(string, "or ");
+        if (portscan || synports[0] != -1 || ackports[0] != -1)
+          strcat(string, "tcp ");
+        strcat(string, ")");
+      } else
+        strcat(string, "icmp6");
+    }
+  } else
+    if (renew)
+      strcpy(string, "ip6");
   
   if (multicast6 != NULL && (enumerate_mac || enumerate_dhcp) && input == NULL && multicast6[0] == 0xff) {
     fprintf(stderr, "Warning: -M/-C options make no sense for multicast addresses and are ignored for these\n");
@@ -814,6 +1022,7 @@ int main(int argc, char *argv[]) {
       memcpy(buf + i*(strlen(tagging)), tagging, strlen(tagging));
   }
 
+  if (debug) printf("Capturing PCAP string: %s\n", string);
   if ((p = thc_pcap_init(interface, string)) == NULL) {
     fprintf(stderr, "Error: could not capture on interface %s with string %s\n", interface, string);
     exit(-1);
@@ -836,10 +1045,13 @@ int main(int argc, char *argv[]) {
   // curr states: 0 = cmdline, 1.. = line no. in input file
   setvbuf(stdout, NULL, _IONBF, 0);
   setvbuf(stderr, NULL, _IONBF, 0);
+
   if (verbose) {
     timeval = time(NULL);
-    printf("Starting alive6 %s (c) 2013 by van Hauser / THC at %s\n", VERSION, ctime(&timeval));
+    printf("Starting alive6 %s (c) 2014 by van Hauser / THC at %s\n", VERSION, ctime(&timeval));
   }
+  if (renew && src6[0] > 0xfd && (multicast6 != NULL && multicast6[0] > 0xfd))
+    prefer = PREFER_LINK;
   while (curr <= list) {
     ok = 1;
     local = -1;
@@ -1167,52 +1379,173 @@ int main(int argc, char *argv[]) {
             curr++;
         }
       } else {
-        if (cip8 < tip8)
-          cip8++;
-        else if (cip7 < tip7) {
-          cip7++;
-          cip8 = fip8;
-        } else if (cip6 < tip6) {
-          cip6++;
-          cip7 = fip7;
-          cip8 = fip8;
-        } else if (cip5 < tip5) {
-          cip5++;
-          cip6 = fip6;
-          cip7 = fip7;
-          cip8 = fip8;
-        } else if (cip4 < tip4) {
-          cip4++;
-          cip5 = fip5;
-          cip6 = fip6;
-          cip7 = fip7;
-          cip8 = fip8;
-        } else if (cip3 < tip3) {
-          cip3++;
-          cip4 = fip4;
-          cip5 = fip5;
-          cip6 = fip6;
-          cip7 = fip7;
-          cip8 = fip8;
-        } else if (cip2 < tip2) {
-          cip2++;
-          cip3 = fip3;
-          cip4 = fip4;
-          cip5 = fip5;
-          cip6 = fip6;
-          cip7 = fip7;
-          cip8 = fip8;
-        } else if (cip1 < tip1) {
-          cip1++;
-          cip2 = fip2;
-          cip3 = fip3;
-          cip4 = fip4;
-          cip5 = fip5;
-          cip6 = fip6;
-          cip7 = fip7;
-          cip8 = fip8;
-        } else
+        inc_step = 1;
+        inc_next = 0;
+//printf("%d S: %04x-%04x=%04x %04x-%04x=%04x %04x-%04x=%04x %04x-%04x=%04x %04x-%04x=%04x %04x-%04x=%04x %04x-%04x=%04x %04x-%04x=%04x\n", ok, fip1, tip1, cip1, fip2, tip2, cip2, fip3, tip3, cip3, fip4, tip4, cip4, fip5, tip5, cip5, fip6, tip6, cip6, fip7, tip7, cip7, fip8, tip8, cip8);
+        if (fip8 != tip8 && (inc_step || inc_next)) {
+          if (inc_step) {
+            inc_step = 0;
+            if (cip8 + step <= tip8) {
+              cip8 += step;
+            } else {
+              cip8 = fip8 + (step + cip8 - tip8);
+              if (step != 256)
+                cip8--;
+              inc_next = 1;
+            }
+          } else {
+            if (cip8 < tip8) {
+              cip8++;
+              inc_next = 0;
+            } else {
+              cip8 = fip8;
+            }
+          }
+        }
+        if (fip7 != tip7 && (inc_step || inc_next)) {
+          if (inc_step) {
+            inc_step = 0;
+            if (cip7 + step <= tip7) {
+              cip7 += step;
+            } else {
+              cip7 = fip7 + (step + cip7 - tip7);
+              if (step != 256)
+                cip7--;
+              inc_next = 1;
+            }
+          } else {
+            if (cip7 < tip7) {
+              cip7++;
+              inc_next = 0;
+            } else {
+              cip7 = fip7;
+            }
+          }
+        }
+        if (fip6 != tip6 && (inc_step || inc_next)) {
+          if (inc_step) {
+            inc_step = 0;
+            if (cip6 + step <= tip6) {
+              cip6 += step;
+            } else {
+              cip6 = fip6 + (step + cip6 - tip6);
+              if (step != 256)
+                cip6--;
+              inc_next = 1;
+            }
+          } else {
+            if (cip6 < tip6) {
+              cip6++;
+              inc_next = 0;
+            } else {
+              cip6 = fip6;
+            }
+          }
+        }
+        if (fip5 != tip5 && (inc_step || inc_next)) {
+          if (inc_step) {
+            inc_step = 0;
+            if (cip5 + step <= tip5) {
+              cip5 += step;
+            } else {
+              cip5 = fip5 + (step + cip5 - tip5);
+              if (step != 256)
+                cip5--;
+              inc_next = 1;
+            }
+          } else {
+            if (cip5 < tip5) {
+              cip5++;
+              inc_next = 0;
+            } else {
+              cip5 = fip5;
+            }
+          }
+        }
+        if (fip4 != tip4 && (inc_step || inc_next)) {
+          if (inc_step) {
+            inc_step = 0;
+            if (cip4 + step <= tip4) {
+              cip4 += step;
+            } else {
+              cip4 = fip4 + (step + cip4 - tip4);
+              if (step != 256)
+                cip4--;
+              inc_next = 1;
+            }
+          } else {
+            if (cip4 < tip4) {
+              cip4++;
+              inc_next = 0;
+            } else {
+              cip4 = fip4;
+            }
+          }
+        }
+        if (fip3 != tip3 && (inc_step || inc_next)) {
+          if (inc_step) {
+            inc_step = 0;
+            if (cip3 + step <= tip3) {
+              cip3 += step;
+            } else {
+              cip3 = fip3 + (step + cip3 - tip3);
+              if (step != 256)
+                cip3--;
+              inc_next = 1;
+            }
+          } else {
+            if (cip3 < tip3) {
+              cip3++;
+              inc_next = 0;
+            } else {
+              cip3 = fip3;
+            }
+          }
+        }
+        if (fip2 != tip2 && (inc_step || inc_next)) {
+          if (inc_step) {
+            inc_step = 0;
+            if (cip2 + step <= tip2) {
+              cip2 += step;
+            } else {
+              cip2 = fip2 + (step + cip2 - tip2);
+              if (step != 256)
+                cip2--;
+              inc_next = 1;
+            }
+          } else {
+            if (cip2 < tip2) {
+              cip2++;
+              inc_next = 0;
+            } else {
+              cip2 = fip2;
+            }
+          }
+        }
+        if (fip1 != tip1 && (inc_step || inc_next)) {
+          if (inc_step) {
+            inc_step = 0;
+            if (cip1 + step <= tip1) {
+              cip1 += step;
+            } else {
+              cip1 = fip1 + (step + cip1 - tip1);
+              if (step != 256)
+                cip1--;
+              inc_next = 1;
+            }
+          } else {
+            if (cip1 < tip1) {
+              cip1++;
+              inc_next = 0;
+            } else {
+              cip1 = fip1;
+            }
+          }
+        }
+
+        if (inc_step || inc_next) // we are done
           ok = 0;
+//printf("%d E: %04x-%04x=%04x %04x-%04x=%04x %04x-%04x=%04x %04x-%04x=%04x %04x-%04x=%04x %04x-%04x=%04x %04x-%04x=%04x %04x-%04x=%04x\n", ok, fip1, tip1, cip1, fip2, tip2, cip2, fip3, tip3, cip3, fip4, tip4, cip4, fip5, tip5, cip5, fip6, tip6, cip6, fip7, tip7, cip7, fip8, tip8, cip8);
 
         cur_dst[0] = cip1 / 256;
         cur_dst[1] = cip1 % 256;
@@ -1280,7 +1613,7 @@ int main(int argc, char *argv[]) {
       fprintf(stderr, "Error: WTF?!\n");
       exit(-1);
     }
-    if (print_only) {
+    if (print_only && ok) {
       p2 = thc_ipv62notation(cur_dst);
       printf("Address: %s\n", p2);
       free(p2);
@@ -1294,7 +1627,7 @@ int main(int argc, char *argv[]) {
     }
     // central dst mac lookup and fast/slow implementation
     no_send = no_send_local;
-    if (ok != 0 && rawmode == 0 && cur_dst != NULL && do_router == 0 && use_dmac == 0) {
+    if (ok != 0 && cur_dst != NULL && do_router == 0 && use_dmac == 0) {
       if (local == -1)
         local = thc_is_dst_local(interface, cur_dst);
       if (local == 0 && slow == 0) {
@@ -1342,6 +1675,12 @@ int main(int argc, char *argv[]) {
       no_send = no_send_remote;
 
     if (ok != 0 && cur_dst != NULL) {
+      if (renew) {
+        while ((src6 = thc_get_own_ipv6(interface, cur_dst, prefer)) == NULL || (prefer == PREFER_GLOBAL && src6[0] > 0xfd)) {
+          fprintf(stderr, "Error: no global IPv6 address found for interface %s, sleeping for 60 seconds and waiting for one! Next check in 60 seconds ...\n", interface);
+          sleep(60);
+        }
+      }
       if (debug)
         printf("DEBUG: sending alive check packets to %s\n", thc_ipv62notation(cur_dst));
       else if (verbose > 2) {
@@ -1350,6 +1689,52 @@ int main(int argc, char *argv[]) {
         free(p2);
       }
       for (nos = 0; nos < no_send; nos++) {     // send -n defined times, default: 1
+        if (synports[0] != -1 || portscan) {
+          i = 0;
+          while ((portscan > 0 && portscan < 65536) || (synports[i] != -1 && i < MAX_PORTS)) {
+            if ((pkt = thc_create_ipv6_extended(interface, prefer, &pkt_len, src6, cur_dst, 0, 0, 0, 0, 0)) == NULL)
+              return -1;
+            if (router6 != NULL)
+              if (thc_add_hdr_route(pkt, &pkt_len, routers, 1) < 0)
+                return -1;
+            if (portscan) {
+              if (thc_add_tcp(pkt, &pkt_len, sp, portscan % 65536, (sp << 16) + sp, 0, TCP_SYN, 5760, 0, NULL, 0, NULL, 0) < 0)
+                return -1;
+              portscan++;
+            } else {
+              if (thc_add_tcp(pkt, &pkt_len, sp, synports[i] % 65536, (sp << 16) + sp, 0, TCP_SYN, 5760, 0, tcp_opt, TCP_OPT_LEN, NULL, 0) < 0)
+                return -1;
+            }
+            if (thc_generate_pkt(interface, smac, mac, pkt, &pkt_len) < 0) {
+              fprintf(stderr, "Error: Can not send packet, exiting ...\n");
+              exit(-1);
+            }
+            if (router6 != NULL) {
+              hdr = (thc_ipv6_hdr *) pkt;
+              thc_send_as_fragment6(interface, src6, cur_dst, NXT_ROUTE, hdr->pkt + 40 + offset, hdr->pkt_len - 40 - offset,
+                                    hdr->pkt_len > 1240 ? 1240 : (((hdr->pkt_len - 40 - offset) / 16) + 1) * 8);
+            } else
+              errcnt = 0;
+              while(thc_send_pkt(interface, pkt, &pkt_len) < 0) {
+                usleep(1);
+                if (errcnt > 0)
+                  usleep(errcnt << 10);
+                else if (errcnt > 10) {
+                  printf("Error: unable to send packet to network, waiting for 60 seconds to recover ...\n");
+                  sleep(60);
+                  errcnt = 0;
+                }
+                errcnt++;
+              }
+            pkt = thc_destroy_packet(pkt);
+            if (waittime)
+              usleep(waittime);
+            i++;
+            while (thc_pcap_check(p, (char *) check_packets, NULL) > 0);
+          }
+          if (portscan)
+            portscan = 1;
+        }
         if (do_ping) {
           if ((pkt = thc_create_ipv6_extended(interface, prefer, &pkt_len, src6, cur_dst, 0, 0, 0, 0, 0)) == NULL)
             return -1;
@@ -1444,42 +1829,6 @@ int main(int argc, char *argv[]) {
             i++;
           }
         }
-        if (synports[0] != -1 || portscan) {
-          i = 0;
-          while ((portscan > 0 && portscan < 65536) || (synports[i] != -1 && i < MAX_PORTS)) {
-            if ((pkt = thc_create_ipv6_extended(interface, prefer, &pkt_len, src6, cur_dst, 0, 0, 0, 0, 0)) == NULL)
-              return -1;
-            if (router6 != NULL)
-              if (thc_add_hdr_route(pkt, &pkt_len, routers, 1) < 0)
-                return -1;
-            if (portscan) {
-              if (thc_add_tcp(pkt, &pkt_len, sp, portscan % 65536, (sp << 16) + sp, 0, TCP_SYN, 5760, 0, NULL, 0, NULL, 0) < 0)
-                return -1;
-              portscan++;
-            } else {
-              if (thc_add_tcp(pkt, &pkt_len, sp, synports[i] % 65536, (sp << 16) + sp, 0, TCP_SYN, 5760, 0, NULL, 0, NULL, 0) < 0)
-                return -1;
-            }
-            if (thc_generate_pkt(interface, smac, mac, pkt, &pkt_len) < 0) {
-              fprintf(stderr, "Error: Can not send packet, exiting ...\n");
-              exit(-1);
-            }
-            if (router6 != NULL) {
-              hdr = (thc_ipv6_hdr *) pkt;
-              thc_send_as_fragment6(interface, src6, cur_dst, NXT_ROUTE, hdr->pkt + 40 + offset, hdr->pkt_len - 40 - offset,
-                                    hdr->pkt_len > 1240 ? 1240 : (((hdr->pkt_len - 40 - offset) / 16) + 1) * 8);
-            } else
-              while(thc_send_pkt(interface, pkt, &pkt_len) < 0)
-                usleep(1);
-            pkt = thc_destroy_packet(pkt);
-            if (waittime)
-              usleep(waittime);
-            i++;
-            while (thc_pcap_check(p, (char *) check_packets, NULL) > 0);
-          }
-          if (portscan)
-            portscan = 1;
-        }
         if (ackports[0] != -1) {
           i = 0;
           while (ackports[i] != -1 && i < MAX_PORTS) {
